/*
 * ConnectedComponentTest.java
 * JUnit based test
 */

package connectedsubgraph;

import matching.*;

import junit.framework.*;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.Edge;
import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.ArrayList;


public class ConnectedComponentTest extends TestCase {
    
    public Graph graph3, graph3_vertices0_and_2, graph15, graph25;
            
    public ConnectedComponentTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
        // ---
        // Test data 3
        // 0 -- 1 -- 2
        // ---
        GraphCreator g = new GraphCreator();
        //graph1 = g.createGraph(g.labels1, g.edges1);
        graph3 = g.createGraph(g.labels3, g.edges3);
        
        String[] labels3_subgraph = {
            // 0-3 vertices
            "node1",        // 0
            //"node2",        // 1
            "node3"         // 2
        };
        graph3_vertices0_and_2 = g.createGraph(labels3_subgraph, null);
        
        graph15 = g.createGraph(g.labels15, g.edges15);
        graph25 = g.createGraph(g.labels25, g.edges25);
    }

    protected void tearDown() throws Exception {
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(ConnectedComponentTest.class);
        
        return suite;
    }

    
    /**
     * Test of getConnectedComponent method, of class connectedsubgraph.ConnectedComponent.
     */
    public void testGetConnectedComponent() {
        System.out.println("getConnectedComponent");
        Vertex v0, v1, v2, v3, v4, v5;
        
        v1 = VertexPairSimilar.getVertexByName(graph3_vertices0_and_2, "node1");
        v3 = VertexPairSimilar.getVertexByName(graph3_vertices0_and_2, "node3");
        ConnectedComponent coco = new ConnectedComponent();
        
        Set<Vertex> result = coco.getConnectedComponent(v1, graph3_vertices0_and_2);
        // there is only one vertex "node1" in the graph "node1", "node3"
        assertEquals(1, result.size());
        
        result = coco.getConnectedComponent(v3, graph3_vertices0_and_2);
        // there is only one vertex "node3" in the graph "node1", "node3"
        assertEquals(1, result.size());
        
        result = coco.getConnectedComponent(null, graph3_vertices0_and_2);
        // there is no null vertices in the graph
        assertEquals(null, result);
        
        // node1 -- node2 -- node3
        v1 = VertexPairSimilar.getVertexByName(graph3, "node1");
        v2 = VertexPairSimilar.getVertexByName(graph3, "node2");
        v3 = VertexPairSimilar.getVertexByName(graph3, "node3");
        result = coco.getConnectedComponent(v3, graph3);
        // all three vertices are connected
        assertEquals(3, result.size());
        
        
        
        /* -----------------------------------------------------------
        Test data 15                  Test data 25

              | 0 Thing                    | 0 Thing
              |                            |
              | 1 Supplier                 | 1 Provider
             /`-.                         /`\ 
           .'    `-.                    .'    \_ 
          /         `-2 Hardware       /        ^--2 Computer
     3 Furniture                     3 Furniture-/ 
                                                      
                 .4 Mobile-Phone                        4 Calculator
                 |                                      
                 .5 Communicator                        5 LED        */ 
        
        // test component 1 (15)
        v3 = VertexPairSimilar.getVertexByName(graph15, "Furniture");
        result = coco.getConnectedComponent(v3, graph15);
        // component 1 has 4 vertices
        assertEquals(4, result.size());
        
        Set<String> str_set = new HashSet<String>();
        for(Vertex v:result) {
            //str_set.add( (String)(((Vertex)o).getUserDatum(GraphComparator.NAME_KEY)) );
            str_set.add( (String)(v.getUserDatum(GraphComparator.NAME_KEY)) );
        }
        assertTrue(str_set.contains("Thing"));
        assertTrue(str_set.contains("Supplier"));
        assertTrue(str_set.contains("Hardware"));
        assertTrue(str_set.contains("Furniture"));
        
        
        // test component 2 (25)
        v3 = VertexPairSimilar.getVertexByName(graph25, "Furniture");
        result = coco.getConnectedComponent(v3, graph25);
        // component 1 has 3 vertices
        assertEquals(4, result.size());
        
        str_set = new HashSet<String>();
        for(Vertex v:result) {
            //str_set.add( (String)(((Vertex)o).getUserDatum(GraphComparator.NAME_KEY)) );
            str_set.add( (String)(v.getUserDatum(GraphComparator.NAME_KEY)) );
        }
        assertTrue(str_set.contains("Thing"));
        assertTrue(str_set.contains("Provider"));
        assertTrue(str_set.contains("Computer"));
        assertTrue(str_set.contains("Furniture"));
    }

    /**
     * Test of getConnectedComponents method, of class connectedsubgraph.ConnectedComponent.
     */
    public void testGetConnectedComponents() {
        System.out.println("getConnectedComponents");
        
        Graph g_cur = null;
        Graph g_all = null;
        ConnectedComponent coco = new ConnectedComponent();
        
        /* -----------------------------------------------------------
        Test data 15                  Test data 25

              | 0 Thing                    | 0 Thing
              |                            |
              | 1 Supplier                 | 1 Provider
             /`-.                         /`\ 
           .'    `-.                    .'    \_ 
          /         `-2 Hardware       /        ^--2 Computer
     3 Furniture                     3 Furniture-/ 
                                                      
                 .4 Mobile-Phone                        4 Calculator
                 |                                      
                 .5 Communicator                        5 LED        */ 
        
        // there are 2 components in data 15
        List<Set<Vertex>> result = coco.getConnectedComponents(graph15);
        assertEquals(2, result.size());
        
        // there are 3 components in data 25
        result = coco.getConnectedComponents(graph25);
        assertEquals(3, result.size());
        
    }
    
}


        // component 1
        /*v0 = VertexPairSimilar.getVertexByName(graph15, "Thing");
        v1 = VertexPairSimilar.getVertexByName(graph15, "Supplier");
        v2 = VertexPairSimilar.getVertexByName(graph15, "Hardware");
        v3 = VertexPairSimilar.getVertexByName(graph15, "Furniture");
        
        // component 2
        v4 = VertexPairSimilar.getVertexByName(graph15, "Mobile-Phone");
        v5 = VertexPairSimilar.getVertexByName(graph15, "Communicator");
        */