/*
 * ConnectedSubgraphTest.java
 * JUnit based test
 */

package connectedsubgraph;

//import matching.UnweightedShortestPathUtil;
import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;

import junit.framework.*;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.graph.impl.SparseGraph;

import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.ArrayList;
import matching.*;

public class ConnectedSubgraphTest extends TestCase {
    
    public Graph    graph3;
    
    public ConnectedSubgraphTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
    }

    protected void tearDown() throws Exception {
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(ConnectedSubgraphTest.class);
        
        return suite;
    }

    /**
     * Test of selectsMinimalVerticesSetByLabels method, of class matching.ConnectedSubgraph.
     */
    public void testSelectsMinimalVerticesSetByLabels() {
        System.out.println("selectsMinimalVerticesSetByLabels");
        
        // ---
        // Test data 3
        // 0 -- 1 -- 2
        // ---
        GraphCreator g = new GraphCreator();
        Graph graph3 = g.createGraph(g.labels3, g.edges3);
        
        String[] labels3_subgraph = {
            // 1-3 vertices
            "node1",
            //"node2",      
            "node3"
        };
        Graph graph3_vertices0_and_2 = g.createGraph(labels3_subgraph, null);
        
        Vertex sub_v0 = VertexPairSimilar.getVertexByName(graph3_vertices0_and_2, "node1");
        Vertex sub_v2 = VertexPairSimilar.getVertexByName(graph3_vertices0_and_2, "node3");
        Vertex v0 = VertexPairSimilar.getVertexByName(graph3, "node1");
        Vertex v1 = VertexPairSimilar.getVertexByName(graph3, "node2");
        Vertex v2 = VertexPairSimilar.getVertexByName(graph3, "node3");
        
        /** Map of subgraph vertices to vertices of the whole graph */
        Map<Vertex,Vertex> vert_subgraph_to_vert;
        /** Map of vertices of the whole graph to vertices of the subgraph */
        Map<Vertex,Vertex> vert_to_vert_subgraph;
        vert_subgraph_to_vert = new HashMap<Vertex,Vertex>();
        vert_to_vert_subgraph = new HashMap<Vertex,Vertex>();
        
        vert_subgraph_to_vert.put(sub_v0,v0);
        vert_subgraph_to_vert.put(sub_v2,v2);
        vert_to_vert_subgraph.put(v0,sub_v0);
        vert_to_vert_subgraph.put(v2,sub_v2);
        
        Graph g_result = ConnectedSubgraph.selectsMinimumVerticesSetByLabels(
                            graph3_vertices0_and_2, graph3,
                            vert_subgraph_to_vert, vert_to_vert_subgraph);
        
        assertEquals(2, g_result.getEdges().size());
        assertEquals(3, g_result.getVertices().size());
        
        Set<String> str_set = new HashSet<String>();
        for(Object o:g_result.getVertices()) {
            str_set.add( (String)(((Vertex)o).getUserDatum(GraphComparator.NAME_KEY)) );
        }
        assertTrue(str_set.contains("node1"));
        assertTrue(str_set.contains("node2"));
        assertTrue(str_set.contains("node3"));
    }
    
    
    /**
     * Test of calcShortestDistances method, of class connectedsubgraph.DistanceComponent.
     * Unconnected graphs
         Test data 15                  
                
              | 0 Thing                
              |                        
              | 1 Supplier             
             /`-.                 
           .'    `-.              
          /         `-2 Hardware  
     3 Furniture                  
                                                      
                 .4 Mobile-Phone  
                 |                                      
                 .5 Communicator  
     *
     * Let's test the graph 15.
     * Vertices set:           selected 2 sets:
     *          0, 3, 4, 5;    (0,1,3) & (4,5): edges:3, vertices:5
     */
    public void testSelectsMinimalVerticesSetByLabels_15() {
        System.out.println("selectsMinimalVerticesSetByLabels_15");
        
        GraphCreator creator = new GraphCreator();
        Graph g15 = creator.createGraph(creator.labels15, creator.edges15);
        UnweightedShortestPath usp = new UnweightedShortestPath(g15);
        //Assert.assertEquals(usp.getDistance((Vertex) id.getVertex(0),(Vertex) id.getVertex(3)).intValue(),2);
        
        Vertex v_thing     = VertexPairSimilar.getVertexByName(g15, "Thing");
        Vertex v_furniture  = VertexPairSimilar.getVertexByName(g15, "Furniture");
        Vertex v_supplier  = VertexPairSimilar.getVertexByName(g15, "Supplier");
        Vertex v_mobile_phone  = VertexPairSimilar.getVertexByName(g15, "Mobile-Phone");
        Vertex v_communicator  = VertexPairSimilar.getVertexByName(g15, "Communicator");
        
        String[] labels3_subgraph = {
            // 1-3 vertices
            "Thing",
            "Furniture",      
            "Mobile-Phone",
            "Communicator"
        };
        GraphCreator g = new GraphCreator();
        Graph sub_graph = g.createGraph(labels3_subgraph, null);
        
        Vertex sub_v_thing   = VertexPairSimilar.getVertexByName(sub_graph, "Thing");
        Vertex sub_v_furniture  = VertexPairSimilar.getVertexByName(sub_graph, "Furniture");
        Vertex sub_v_mobile_phone = VertexPairSimilar.getVertexByName(sub_graph, "Mobile-Phone");
        Vertex sub_v_communicator = VertexPairSimilar.getVertexByName(sub_graph, "Communicator");
        
        /** Map of subgraph vertices to vertices of the whole graph */
        Map<Vertex,Vertex> vert_subgraph_to_vert;
        /** Map of vertices of the whole graph to vertices of the subgraph */
        Map<Vertex,Vertex> vert_to_vert_subgraph;
        vert_subgraph_to_vert = new HashMap<Vertex,Vertex>();
        vert_to_vert_subgraph = new HashMap<Vertex,Vertex>();
        
        vert_subgraph_to_vert.put(sub_v_thing,          v_thing);
        vert_subgraph_to_vert.put(sub_v_furniture,      v_furniture);
        vert_subgraph_to_vert.put(sub_v_mobile_phone,   v_mobile_phone);
        vert_subgraph_to_vert.put(sub_v_communicator,   v_communicator);
        
        vert_to_vert_subgraph.put(v_thing,              sub_v_thing);
        vert_to_vert_subgraph.put(v_furniture,          sub_v_furniture);
        vert_to_vert_subgraph.put(v_mobile_phone,       sub_v_mobile_phone);
        vert_to_vert_subgraph.put(v_communicator,       sub_v_communicator);
        
        Graph g_result = ConnectedSubgraph.selectsMinimumVerticesSetByLabels(sub_graph, g15,
                                vert_subgraph_to_vert, vert_to_vert_subgraph);
        
        assertEquals(3, g_result.getEdges().size());
        assertEquals(5, g_result.getVertices().size());
        
        Set<String> str_set = new HashSet<String>();
        for(Object o:g_result.getVertices()) {
            str_set.add( (String)(((Vertex)o).getUserDatum(GraphComparator.NAME_KEY)) );
        }
        assertTrue(str_set.contains("Thing"));
        assertTrue(str_set.contains("Supplier"));
        assertTrue(str_set.contains("Furniture"));
        assertTrue(str_set.contains("Mobile-Phone"));
        assertTrue(str_set.contains("Communicator"));
    }
    
    /**
     * Test of calcShortestDistances method, of class connectedsubgraph.DistanceComponent.
     * Unconnected graphs
         Test data 15                  Test data 25
                
              | 0 Thing                    | 0 Thing
              |                            |
              | 1 Supplier                 | 1 Provider
             /`-.                         /`\ 
           .'    `-.                    .'    \_ 
          /         `-2 Hardware       /        ^--2 Computer
     3 Furniture                     3 Furniture-/ 
                                                      
                 .4 Mobile-Phone                        4 Calculator
                 |                                      
                 .5 Communicator                        5 LED
     *
     * Let's test the graph 25.
     * Vertices set:           selected 3 sets: 
     *          3, 2, 4, 5;    (3, 2) 4, 5: edges:1, vertices:4
     *
     *todo: test with non-unique names of vertices.
     */
    public void testSelectsMinimalVerticesSetByLabels_25() {
        System.out.println("selectsMinimalVerticesSetByLabels_25");
        
        GraphCreator creator = new GraphCreator();
        Graph g25 = creator.createGraph(creator.labels25, creator.edges25);
        UnweightedShortestPath usp = new UnweightedShortestPath(g25);
        //Assert.assertEquals(usp.getDistance((Vertex) id.getVertex(0),(Vertex) id.getVertex(3)).intValue(),2);
        
        Vertex v_furniture  = VertexPairSimilar.getVertexByName(g25, "Furniture");
        Vertex v_computer  = VertexPairSimilar.getVertexByName(g25, "Computer");
        Vertex v_calculator  = VertexPairSimilar.getVertexByName(g25, "Calculator");
        Vertex v_LED  = VertexPairSimilar.getVertexByName(g25, "LED");
        
        String[] labels3_subgraph = {
            // 1-3 vertices
            "Computer",
            "Furniture",      
            "Calculator",
            "LED"
        };
        GraphCreator g = new GraphCreator();
        Graph sub_graph = g.createGraph(labels3_subgraph, null);
        
        Vertex sub_v_computer   = VertexPairSimilar.getVertexByName(sub_graph, "computer");
        Vertex sub_v_furniture  = VertexPairSimilar.getVertexByName(sub_graph, "Furniture");
        Vertex sub_v_calculator = VertexPairSimilar.getVertexByName(sub_graph, "Calculator");
        Vertex sub_v_LED = VertexPairSimilar.getVertexByName(sub_graph, "LED");
        
        /** Map of subgraph vertices to vertices of the whole graph */
        Map<Vertex,Vertex> vert_subgraph_to_vert;
        /** Map of vertices of the whole graph to vertices of the subgraph */
        Map<Vertex,Vertex> vert_to_vert_subgraph;
        vert_subgraph_to_vert = new HashMap<Vertex,Vertex>();
        vert_to_vert_subgraph = new HashMap<Vertex,Vertex>();
        
        vert_subgraph_to_vert.put(sub_v_computer,       v_computer);
        vert_subgraph_to_vert.put(sub_v_furniture,      v_furniture);
        vert_subgraph_to_vert.put(sub_v_calculator,     v_calculator);
        vert_subgraph_to_vert.put(sub_v_LED,   v_LED);
        
        vert_to_vert_subgraph.put(v_computer,           sub_v_computer);
        vert_to_vert_subgraph.put(v_furniture,          sub_v_furniture);
        vert_to_vert_subgraph.put(v_calculator,         sub_v_calculator);
        vert_to_vert_subgraph.put(v_LED,       sub_v_LED);
        
        Graph g_result = ConnectedSubgraph.selectsMinimumVerticesSetByLabels(sub_graph, g25,
                                vert_subgraph_to_vert, vert_to_vert_subgraph);
        
        assertEquals(1, g_result.getEdges().size());
        assertEquals(4, g_result.getVertices().size());
        
        Set<String> str_set = new HashSet<String>();
        for(Object o:g_result.getVertices()) {
            str_set.add( (String)(((Vertex)o).getUserDatum(GraphComparator.NAME_KEY)) );
        }
        assertTrue(str_set.contains("Computer"));
        assertTrue(str_set.contains("Furniture"));
        assertTrue(str_set.contains("Calculator"));
        assertTrue(str_set.contains("LED"));
    }
}
