package edu.uci.ics.jung.io;

import junit.framework.*;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.InputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.InputSource;
import edu.uci.ics.jung.exceptions.FatalException;
import edu.uci.ics.jung.graph.DirectedEdge;
import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.UndirectedEdge;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.decorators.Indexer;
import edu.uci.ics.jung.utils.Pair;
import edu.uci.ics.jung.utils.PredicateUtils;
import edu.uci.ics.jung.utils.UserDataContainer;

import edu.uci.ics.jung.graph.decorators.StringLabeller;

/**
 * @author Scott White
 * @author Andre Krizhanovsky
 */
public class GraphMLFileTest extends TestCase {
    
    public GraphMLFileTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
    }

    protected void tearDown() throws Exception {
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(GraphMLFileTest.class);
        
        return suite;
    }

    public void testLoad() {
        System.out.println("load");
        String testFilename = "toy_graph.ml";

        Graph graph = loadGraph(testFilename);

        Assert.assertEquals(graph.numVertices(),3);
        Assert.assertEquals(graph.numEdges(),3);

        StringLabeller labeller = StringLabeller.getLabeller(graph);

        Vertex joe = labeller.getVertex("1");
        Vertex bob = labeller.getVertex("2");
        Vertex sue = labeller.getVertex("3");
        Assert.assertEquals(joe.getUserDatum("name"),"Joe");
        Assert.assertEquals(bob.getUserDatum("name"),"Bob");
        Assert.assertEquals(sue.getUserDatum("name"),"Sue");

        Assert.assertTrue(bob.isPredecessorOf(joe));
        Assert.assertTrue(joe.isPredecessorOf(bob));
        Assert.assertTrue(joe.isPredecessorOf(sue));
        Assert.assertFalse(sue.isPredecessorOf(joe));
        Assert.assertFalse(bob.isPredecessorOf(sue));
        Assert.assertFalse(sue.isPredecessorOf(bob));

        File testFile = new File(testFilename);
        testFile.delete();
    }

    private Graph loadGraph(String testFilename) {
        try {
            BufferedWriter writer = new BufferedWriter(new FileWriter(testFilename));
            writer.write("<?xml version=\"1.0\" encoding=\"iso-8859-1\" ?>\n");
             writer.write("<?meta name=\"GENERATOR\" content=\"XML::Smart 1.3.1\" ?>\n");
            writer.write("<graph edgedefault=\"directed\">\n");
            writer.write("<node id=\"1\" name=\"Joe\"/>\n");
            writer.write("<node id=\"2\" name=\"Bob\"/>\n");
            writer.write("<node id=\"3\" name=\"Sue\"/>\n");
            writer.write("<edge source=\"1\" target=\"2\"/>\n");
            writer.write("<edge source=\"2\" target=\"1\"/>\n");
            writer.write("<edge source=\"1\" target=\"3\"/>\n");
             writer.write("</graph>\n");
            writer.close();

        } catch (IOException ioe) {
        }

        GraphMLFile graphmlFile = new GraphMLFile();
        Graph graph = graphmlFile.load(testFilename);
        return graph;
    }
    
    /*public void testLoadGraphCollection() {
        System.out.println("loadGraphCollection");
        
        String dirName = "";
        FilenameFilter filter = null;
        GraphMLFile instance = new GraphMLFile();
        
        List expResult = null;
        List result = instance.loadGraphCollection(dirName, filter);
        assertEquals(expResult, result);
        
        // TODO review the generated test code and remove the default call to fail.
        fail("The test case is a prototype.");
    }*/

    
    /*public void testSave() {
        Graph g = null;
        String filename = "";
        GraphMLFile instance = new GraphMLFile();
        
        instance.save(g, filename);
        
        // TODO review the generated test code and remove the default call to fail.
        fail("The test case is a prototype.");
    }*/

    public void testSave() {
        System.out.println("save");
        
        String testFilename = "toy_graph.ml";
        Graph oldGraph = loadGraph(testFilename);
        GraphMLFile graphmlFile = new GraphMLFile();
        String newFilename = testFilename + "_save";
        graphmlFile.save(oldGraph,newFilename);
		Graph newGraph = graphmlFile.load(newFilename);
        Assert.assertEquals(oldGraph.numVertices(),newGraph.numVertices());
        Assert.assertEquals(oldGraph.numEdges(),newGraph.numEdges());
        File testFile = new File(testFilename);
        testFile.delete();
        File newFile = new File(newFilename);
//        newFile.delete();
    }

    /**
     * Test of setGraphMLFileHandler method, of class edu.uci.ics.jung.io.GraphMLFile.
     */
    /*public void testSetGraphMLFileHandler() {
        System.out.println("setGraphMLFileHandler");
        
        GraphMLFileHandler fileHandler = null;
        GraphMLFile instance = new GraphMLFile();
        
        instance.setGraphMLFileHandler(fileHandler);
        
        // TODO review the generated test code and remove the default call to fail.
        fail("The test case is a prototype.");
     }*/
    
}
