package matching;

import junit.framework.*;

public class LevenshteinDistanceTest extends TestCase {
    
    public LevenshteinDistanceTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
    }

    protected void tearDown() throws Exception {
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(LevenshteinDistanceTest.class);
        
        return suite;
    }

    /**
     * Test of computeLevenshteinDistance method, of class topindex.LevenshteinDistance.
     */
    public void testComputeLevenshteinDistanceArray() {
        System.out.println("computeLevenshteinDistance String[], String[]");
        
        String[] words1 = {"apple", "verylongword"};
        String[] words2 = {"appl"};
        
        LevenshteinDistance lev = new LevenshteinDistance();
        lev.use_dist_matrix = true;
        lev.setContstraintType(LevenshteinDistance.TYPE_HARD);
        
        // test identical words
        int result = lev.computeLevenshteinDistance(words1, words1);
        assertEquals(0, result);
        
        result = lev.computeLevenshteinDistance(words1, words2);
        assertEquals(1, result);
        
        String[] words3 = {"Safety", "SecondBuzzyWord"};
        String[] words4 = {"Health"};
        result = lev.computeLevenshteinDistance(words3, words4);
        assertEquals(-1, result);
        
        lev.use_dist_matrix = false;
        result = lev.computeLevenshteinDistance(words3, words4);
        assertEquals(5, result);
        
        // test stop word "units"
        String[] words_with_stop = {"appleBuzzyWordVeryLong", "units"};
        String[] words5 = {"unitz"};
        
        lev.use_dist_matrix = true;
        result = lev.computeLevenshteinDistance(words_with_stop , words5);
        assertEquals(-1, result);
        
        // test stop words 2
        String[] words_with_stop2 = {"gluunitxy", "units"};
        String[] words6 = {"gluunit"};
        
        result = lev.computeLevenshteinDistance(words_with_stop2 , words6);
        assertEquals(2, result);
        
        // test empty words
        System.out.println("computeLevenshteinDistance String[], String[]: test empty words");
        String[] words_empty = {"", null};
        String[] words7 = {"gluunit"};
        
        result = lev.computeLevenshteinDistance(words_empty, words7);
        assertEquals(-1, result);        
    }
    
    public void testComputeLevenshteinDistanceStrings() {
        System.out.println("computeLevenshteinDistance Char[], Char[]");
        
        LevenshteinDistance lev = new LevenshteinDistance();
        lev.setContstraintType(LevenshteinDistance.TYPE_SOFT);
        lev.use_dist_matrix = true;
        int result = lev.computeLevenshteinDistance("Supplier", "WordVeryUnsimilarTo");
        assertEquals(-1, result);
        
        result = lev.computeLevenshteinDistance("Supplier", "Provider");
        assertEquals(6, result);
        
        // test stop word units
        result = lev.computeLevenshteinDistance("unitx", "units");
        assertEquals(-1, result);
    }
    
}
