package matching;

import junit.framework.*;
import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;
import java.util.List;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import edu.uci.ics.jung.algorithms.importance.*;
import edu.uci.ics.jung.utils.MutableDouble;
import edu.uci.ics.jung.utils.UserData;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.impl.SparseGraph;
import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.graph.Element;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.ArchetypeVertex;
import edu.uci.ics.jung.graph.decorators.DefaultToolTipFunction;
import edu.uci.ics.jung.graph.decorators.EdgeShape;
import edu.uci.ics.jung.graph.decorators.EllipseVertexShapeFunction;
import edu.uci.ics.jung.graph.decorators.PickableVertexPaintFunction;
import edu.uci.ics.jung.graph.decorators.VertexIconAndShapeFunction;
import edu.uci.ics.jung.graph.decorators.VertexStringer;
import edu.uci.ics.jung.graph.impl.UndirectedSparseEdge;
import edu.uci.ics.jung.graph.impl.DirectedSparseEdge;
import edu.uci.ics.jung.graph.impl.SparseVertex;
import edu.uci.ics.jung.visualization.DefaultGraphLabelRenderer;
import edu.uci.ics.jung.visualization.FRLayout;
import edu.uci.ics.jung.visualization.GraphZoomScrollPane;
import edu.uci.ics.jung.visualization.PluggableRenderer;
import edu.uci.ics.jung.visualization.ShapePickSupport;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.CrossoverScalingControl;
import edu.uci.ics.jung.visualization.control.DefaultModalGraphMouse;
import edu.uci.ics.jung.visualization.control.ModalGraphMouse;
import edu.uci.ics.jung.visualization.control.ScalingControl;

public class SimVertConnectByEdgeTest extends TestCase {

    private Graph graph;
    
    /** Whether to enable visual effects (e.g. draw edge labels) if true,
     * skip all visual functions if only calculations to be done. */
    public boolean enable_visual;
    
    /** the visual component and renderer for the graph */
    VisualizationViewer vv;
    
    UnicodeVertexStringer   u_vertexstringer;
    UnicodeEdgeStringer     u_edgestringer;
    
    public SimVertConnectByEdgeTest(String testName) {
        super(testName);
        enable_visual = false;
    }

    protected void setUp() throws Exception {   
    }

    protected void tearDown() throws Exception {
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(SimVertConnectByEdgeTest.class);
        return suite;
    }
    
    public void testCalcNodesStringSimilarity() {
        System.out.println("calcNodesStringSimilarity");
        
        SimVertConnectByEdge instance = new SimVertConnectByEdge();
        GraphCreator gd = new GraphCreator();
        
        // let's sub_graph1 contains v[0]="Supplier"
        // let's sub_graph2 contains v[1,2,3]="Suppliez", "Spplieer", "WordVeryUnsimilarTo"
        
        String[] _labels1 = {
            // sub_graph1 vertices
            "Supplier"        // 0
        };
                                    //      Edges   Levenshtein_distance_to_"Supplier"  
        String[] _labels2 = {
            // sub_graph2 vertices          
            "Suppliez",             // 1    -       1
            "Spplieer",             // 2    1->2    2
            "WordVeryUnsimilarTo"   // 3    1->3    -1
        };
        int[][] edges2 = { {0,1}, {0,2}};
        
        Graph sub_graph1 = gd.createGraph(_labels1, null);
        //Graph sub_graph2 = gd.createGraph(_labels2, edges2);
        Graph _graph = gd.addGraph(sub_graph1, _labels2, edges2);
        
        Vertex v0 = VertexPairSimilar.getVertexByName(_graph, "Supplier");
        Vertex v1 = VertexPairSimilar.getVertexByName(_graph, "Suppliez");
        Vertex v2 = VertexPairSimilar.getVertexByName(_graph, "Spplieer");
        Vertex v3 = VertexPairSimilar.getVertexByName(_graph, "WordVeryUnsimilarTo");
        
        // test function
        LevenshteinDistance lev = new LevenshteinDistance();
        lev.use_dist_matrix = true;
        lev.setContstraintType(LevenshteinDistance.TYPE_SOFT);
        //instance.calcNodesStringSimilarity(_graph, sub_graph1, sub_graph2, lev);
        instance.calcNodesStringSimilarity(_graph, lev);
                
        // Supplier <-> Suppliez
        Edge e;
        double sim;
        e = v0.findEdge(v1);
        assertTrue(null != e);
        sim = (Double)e.getUserDatum(GraphComparator.SIM_KEY);
        assertTrue(VertexPairSimilar.equalsDoubles(1.0, sim));
        
        // Supplier <-> Smarthpone
        e = v0.findEdge(v2);
        assertTrue(null != e);
        sim = (Double)e.getUserDatum(GraphComparator.SIM_KEY);
        assertTrue(VertexPairSimilar.equalsDoubles(2.0, sim));
        
        // Supplier <-> WordVeryUnsimilarTo
        e = v0.findEdge(v3);
        assertTrue(null == e);
    }

    
    
    private void loadImages(Vertex[] vertices, Map imageMap) {
        
        ImageIcon icon_class = null;
        ImageIcon icon_attr = null;
        try {
            icon_class  = new ImageIcon(getClass().getResource("/class.png"));
            icon_attr   = new ImageIcon(getClass().getResource("/attr.png"));
        } catch(Exception ex) {
            System.err.println("You need class_icons.jar in your classpath to see the class, attribute icons.");
        }
        
        for(Vertex v:vertices) {
            imageMap.put(v,icon_class);
        }
    }
        
    public void draw()
    {   
        Vertex[] v = (Vertex[])(graph.getVertices()).toArray(new Vertex[0]);
        
        PluggableRenderer pr = new PluggableRenderer();
        u_vertexstringer = new UnicodeVertexStringer();
        pr.setVertexStringer(u_vertexstringer);
        pr.setVertexPaintFunction(new PickableVertexPaintFunction(pr, Color.lightGray, Color.white,  Color.yellow));
        pr.setGraphLabelRenderer(new DefaultGraphLabelRenderer(Color.cyan, Color.cyan));
        VertexIconAndShapeFunction dvisf =
            new VertexIconAndShapeFunction(new EllipseVertexShapeFunction());
        pr.setVertexShapeFunction(dvisf);
        pr.setVertexIconFunction(dvisf);
        loadImages(v, dvisf.getIconMap());
        vv =  new VisualizationViewer(new FRLayout(graph), pr);
        vv.setPickSupport(new ShapePickSupport());
        pr.setEdgeShapeFunction(new EdgeShape.QuadCurve());
        vv.setBackground(Color.white);
        
        u_edgestringer = new UnicodeEdgeStringer ();
        pr.setEdgeStringer(u_edgestringer);
        //pr.setEdgePaintFunction(new PickableEdgePaintFunction(pr, Color.black, Color.cyan));

        // add my listener for ToolTips
        vv.setToolTipFunction(new DefaultToolTipFunction());
        
        // create a frome to hold the graph
        final JFrame frame = new JFrame();
        Container content = frame.getContentPane();
        final GraphZoomScrollPane panel = new GraphZoomScrollPane(vv);
        content.add(panel);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        final ModalGraphMouse gm = new DefaultModalGraphMouse();
        vv.setGraphMouse(gm);
        
        final ScalingControl scaler = new CrossoverScalingControl();

        JButton plus = new JButton("+");
        plus.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                scaler.scale(vv, 1.1f, vv.getCenter());
            }
        });
        JButton minus = new JButton("-");
        minus.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                scaler.scale(vv, 0.9f, vv.getCenter());
            }
        });

        JCheckBox lo = new JCheckBox("Show Node Labels");
        lo.addItemListener(new ItemListener(){
            public void itemStateChanged(ItemEvent e) {
                u_vertexstringer.setShowLabels(e.getStateChange() == ItemEvent.SELECTED);
                vv.repaint();
            }
        });
        lo.setSelected(true);
        
        JCheckBox loe = new JCheckBox("Show Edge Labels");
        loe.addItemListener(new ItemListener(){
            public void itemStateChanged(ItemEvent e) {
                u_edgestringer.setShowLabels(e.getStateChange() == ItemEvent.SELECTED);
                vv.repaint();
            }
        });
        loe.setSelected(true);
        
        JPanel controls = new JPanel();
        controls.add(plus);
        controls.add(minus);
        controls.add(lo);
        controls.add(loe);
        controls.add(((DefaultModalGraphMouse) gm).getModeComboBox());
        content.add(controls, BorderLayout.SOUTH);

        frame.pack();
        frame.setVisible(true);
    }
    
    /** 1) Draws all similar (by Levenshtein distance) vertices connected (many to many)
     *  2) Draws similar vertices connected (one to one)
     */
    public void testDrawConnectedEdges() {
        System.out.println("drawConnectedEdges");
        
        enable_visual = true;
        
        //Load load = new Load();
        //Graph graph = load.loadTestStaticGraph();
        GraphCreator gd = new GraphCreator();
        Graph g1 = gd.createGraph(gd.labels1, gd.edges1);
        Graph g2 = gd.createGraph(gd.labels2, gd.edges2);
        //gc.calcNodesStringSimilarity(g1, g2, lev);
        graph   = gd.addGraph(g1, gd.labels2, gd.edges2);
        
        // calculates initial similarity
        LevenshteinDistance lev = new LevenshteinDistance();
        lev.use_dist_matrix = true;
        lev.setContstraintType(LevenshteinDistance.TYPE_SOFT);
        
        SimVertConnectByEdge sf = new SimVertConnectByEdge();
        GraphComparator gc = new GraphComparator();
        
        boolean b = false;
        if(b)
        { // 1) 
            sf.calcNodesStringSimilarity(graph, lev);
        } else {
            // 2)
            gc.calcNodesStringSimilarity(g1, g2, lev);

            for(Vertex v1:gc.similar_v1_v2.keySet()) {
                Vertex v2 = gc.similar_v1_v2.get(v1);
                String name1 = (String)v1.getUserDatum(GraphComparator.NAME_KEY);
                String name2 = (String)v2.getUserDatum(GraphComparator.NAME_KEY);
                
                Vertex v1_in_graph = VertexPairSimilar.getVertexByName(graph, name1, "1");
                Vertex v2_in_graph = VertexPairSimilar.getVertexByName(graph, name2, null);
                
                Integer sim_key = (Integer)v1.getUserDatum(GraphComparator.SIM_KEY);
                Edge e = graph.addEdge(new UndirectedSparseEdge(v1_in_graph, v2_in_graph));
                e.addUserDatum(GraphComparator.SIM_KEY, (double)sim_key, UserData.SHARED);
            }
        }
        
        draw();
        
        //int i = 0; while(i<1) {i=0;};
    }
}
