package matching;

import junit.framework.*;

import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;
import edu.uci.ics.jung.graph.decorators.Indexer;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.ArchetypeVertex;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.graph.UndirectedGraph;
import edu.uci.ics.jung.graph.impl.UndirectedSparseGraph;
import edu.uci.ics.jung.utils.UserData;
import edu.uci.ics.jung.utils.GraphUtils;

import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;
import java.util.TreeSet;

public class UnweightedShortestPathUtilTest extends TestCase {
    
    public UnweightedShortestPathUtilTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
    }

    protected void tearDown() throws Exception {
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(UnweightedShortestPathUtilTest.class);
        
        return suite;
    }

    public void testGetPath() {
        System.out.println("getPath");

        UndirectedGraph ug = new UndirectedSparseGraph();
        GraphUtils.addVertices(ug,7);
        Indexer id = Indexer.getIndexer(ug);
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(0),(Vertex) id.getVertex(1));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(1),(Vertex) id.getVertex(2));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(2),(Vertex) id.getVertex(3));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(0),(Vertex) id.getVertex(4));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(4),(Vertex) id.getVertex(3));
        
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(5),(Vertex) id.getVertex(6));

        UnweightedShortestPath usp = new UnweightedShortestPath(ug);
        
        // 1) test get path between unconnected vertices 5 and 0
        Vertex source = (Vertex) id.getVertex(5);
        Vertex dest = (Vertex) id.getVertex(0);
        List<Vertex> result = UnweightedShortestPathUtil.getPath(source, dest, usp);
        assertNull(result);
        
        // 2) shortest path from 0 to 3 contains three vertices (3,4,0)
        source = (Vertex) id.getVertex(0);
        dest = (Vertex) id.getVertex(3);
        result = UnweightedShortestPathUtil.getPath(source, dest, usp);
        assertNotNull(result);
        assertEquals(3, result.size());
        assertEquals((Vertex) id.getVertex(3), result.get(0));
        assertEquals((Vertex) id.getVertex(4), result.get(1));
        assertEquals((Vertex) id.getVertex(0), result.get(2));
        
        
        
        // test by GraphCreator.labels3&4, edges3&4 data
        GraphCreator creator = new GraphCreator();
        Graph g1 = creator.createGraph(creator.labels3, creator.edges3);
        Graph g2 = creator.createGraph(creator.labels4, creator.edges4);
        
        LevenshteinDistance lev = new LevenshteinDistance();
        lev.use_dist_matrix = true;
        lev.setContstraintType(LevenshteinDistance.TYPE_SOFT);
        GraphComparator gc_source = new GraphComparator();
        gc_source.calcNodesStringSimilarity(g1, g2, lev);
        
        UnweightedShortestPath usp1 = new UnweightedShortestPath(g1);
        UnweightedShortestPath usp2 = new UnweightedShortestPath(g2);
        
        //Map<Vertex,VertexPairSimilar> pair_similar = new HashMap<Vertex,VertexPairSimilar>();
       
        Vertex v1 = VertexPairSimilar.getVertexByName(g1, "node1");
        Vertex v2 = VertexPairSimilar.getVertexByName(g2, "node1");
        
        Map<Vertex,Integer> v1_path_len = UnweightedShortestPathUtil.getPath(v1, gc_source.similar_v1_v2.keySet(), usp1);
        Map<Vertex,Integer> v2_path_len = UnweightedShortestPathUtil.getPath(v2, gc_source.similar_v2_v1.keySet(), usp2);
        Map<Vertex,Integer> v1_path_len_real = UnweightedShortestPathUtil.getPathReal(v1, gc_source.similar_v1_v2.keySet(), usp1);
        Map<Vertex,Integer> v2_path_len_real = UnweightedShortestPathUtil.getPathReal(v2, gc_source.similar_v2_v1.keySet(), usp2);
        
        VertexPairSimilar pair = new VertexPairSimilar(v1, v2, 
                    (Integer)v1.getUserDatum(GraphComparator.SIM_KEY), 
                    gc_source.similar_v1_v2, gc_source.similar_v2_v1,
                    v1.numNeighbors(), v2.numNeighbors(), 
                    v1_path_len, v2_path_len,
                    v1_path_len_real, v2_path_len_real);
        assertEquals(0.5, pair.getSimilarity());
        
    }
    
    
    /* See test Graph 7 above or below. */
    public void testGetShortestPath () {
        System.out.println("testgetShortestPath between sets of vertices");
        
        UndirectedGraph ug = new UndirectedSparseGraph();
        GraphUtils.addVertices(ug,8);
        Indexer id = Indexer.getIndexer(ug);
        for(int i=0;i<8;i++) {
            ((Vertex) id.getVertex(i)).addUserDatum(GraphComparator.NAME_KEY, i, UserData.SHARED);
        }
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(0),(Vertex) id.getVertex(1));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(1),(Vertex) id.getVertex(2));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(2),(Vertex) id.getVertex(3));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(0),(Vertex) id.getVertex(4));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(4),(Vertex) id.getVertex(3));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(1),(Vertex) id.getVertex(7));
        
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(5),(Vertex) id.getVertex(6));
        UnweightedShortestPath usp = new UnweightedShortestPath(ug);
        
        // [1] there is shortest path (0,1,7) between sets (5,7) and (0,3)
        Set<Vertex> set1 = new HashSet<Vertex>();
        Set<Vertex> set2 = new HashSet<Vertex>();
        set1.add ((Vertex) id.getVertex(5));
        set1.add ((Vertex) id.getVertex(7));
        set2.add ((Vertex) id.getVertex(0));
        set2.add ((Vertex) id.getVertex(3));
        
        Vertex[] path = UnweightedShortestPathUtil.getShortestPath(set1, null, usp);
        assertEquals(null, path);
        path = UnweightedShortestPathUtil.getShortestPath(set1, set2, usp);
        assertTrue(null != path);
        assertTrue(3 == path.length);               //assertTrue(1 == path.size());
        assertTrue(path[0] == id.getVertex(0));     //assertTrue(path.contains(id.getVertex(0)));
        assertTrue(path[1] == id.getVertex(1));     //assertTrue(path.contains(id.getVertex(1)));
        assertTrue(path[2] == id.getVertex(7));     //assertTrue(path.contains(id.getVertex(7)));
        
        // [2] there is no path between sets (0,1,2) and (5,6)
        set1 = new HashSet<Vertex>();
        set2 = new HashSet<Vertex>();
        set1.add ((Vertex) id.getVertex(0));
        set1.add ((Vertex) id.getVertex(1));
        set1.add ((Vertex) id.getVertex(2));
        set2.add ((Vertex) id.getVertex(5));
        set2.add ((Vertex) id.getVertex(6));
        
        path = UnweightedShortestPathUtil.getShortestPath(set1, set2, usp);
        assertTrue(0 == path.length);               //assertTrue(0 == path.size());
        
        // [3] there are two shortest paths (4,0,1,7) or (3,2,1,7) between sets (3,4) and (7)
        set1 = new HashSet<Vertex>();
        set2 = new HashSet<Vertex>();
        set1.add ((Vertex) id.getVertex(3));
        set1.add ((Vertex) id.getVertex(4));
        set2.add ((Vertex) id.getVertex(7));
        
        path = UnweightedShortestPathUtil.getShortestPath(set1, set2, usp);
        assertTrue(4 == path.length);               //assertTrue(4 == path.size());
    }
    
    /* Test Graph 7 (Created in JavE)
     
           0  /______________(1)___________7          5|
            .'                 |                       |
           /                   |                       |
         .'                    |                       |
        X(4)                   |                       |
         `.                    |                       |
           \                   |                       |
            `.3               2|                       |
              \'''''''''''''''''                      6|
     */
    public void testGetPathLenWithListConstraints () {
        System.out.println("testGetPathLen with list of constraints");
        
        UndirectedGraph ug = new UndirectedSparseGraph();
        GraphUtils.addVertices(ug,8);
        Indexer id = Indexer.getIndexer(ug);
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(0),(Vertex) id.getVertex(1));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(1),(Vertex) id.getVertex(2));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(2),(Vertex) id.getVertex(3));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(0),(Vertex) id.getVertex(4));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(4),(Vertex) id.getVertex(3));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(1),(Vertex) id.getVertex(7));
        
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(5),(Vertex) id.getVertex(6));
        UnweightedShortestPath usp = new UnweightedShortestPath(ug);
        
        //Set<ArchetypeVertex> v_constr = new TreeSet<ArchetypeVertex>();
        //Set<Vertex> v_constr = new TreeSet<Vertex>();
        //Set<Vertex> v_constr = new HashSet<Vertex>();
        List<Vertex> v_constr = new ArrayList<Vertex>();
        
        v_constr.add((Vertex) id.getVertex(1));
        v_constr.add((Vertex) id.getVertex(4));
        //Vertex v4 = (Vertex) id.getVertex(4);
        //v_constr.add(v4);
        
        // [1] there is no path between unconnected vertices 5 and 0
        Vertex source = (Vertex) id.getVertex(5);
        Vertex dest = (Vertex) id.getVertex(0);
        int result = UnweightedShortestPathUtil.getPathLen(source, dest, usp, v_constr);
        assertEquals(-1, result);
        
        // [2] shortest path from 0 to 3 contains three vertices (3,4,0), but 4 is constraint
        source = (Vertex) id.getVertex(0);
        dest = (Vertex) id.getVertex(3);
        result = UnweightedShortestPathUtil.getPathLen(source, dest, usp, v_constr);
        assertEquals(-1, result);
        
        // [3] shortest path from 4 to 1 contains three vertices (1,0,4)
        source = (Vertex) id.getVertex(4);
        dest = (Vertex) id.getVertex(1);
        result = UnweightedShortestPathUtil.getPathLen(source, dest, usp, v_constr);
        assertEquals(2, result);
        
        // [4] shortest path from 3 to 7 contains four vertices (7,1,2,3), but 1 is constraint
        source = (Vertex) id.getVertex(3);
        dest = (Vertex) id.getVertex(7);
        result = UnweightedShortestPathUtil.getPathLen(source, dest, usp, v_constr);
        assertEquals(-1, result);
        
        // [5] shortest path from 3 to 1 contains three vertices (1,2,3)
        source = (Vertex) id.getVertex(3);
        dest = (Vertex) id.getVertex(1);
        result = UnweightedShortestPathUtil.getPathLen(source, dest, usp, v_constr);
        assertEquals(2, result);
    }
    
    /* See test Graph 7 above. */
    public void testGetPathLen () {
        System.out.println("testGetPathLen from vertex to vertex");
    /* See test Graph 7 above. */    
        UndirectedGraph ug = new UndirectedSparseGraph();
        GraphUtils.addVertices(ug,8);
        Indexer id = Indexer.getIndexer(ug);
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(0),(Vertex) id.getVertex(1));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(1),(Vertex) id.getVertex(2));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(2),(Vertex) id.getVertex(3));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(0),(Vertex) id.getVertex(4));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(4),(Vertex) id.getVertex(3));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(1),(Vertex) id.getVertex(7));
        
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(5),(Vertex) id.getVertex(6));
        UnweightedShortestPath usp = new UnweightedShortestPath(ug);
        
        // [1] there is no path between unconnected vertices 5 and 0
        Vertex source = (Vertex) id.getVertex(5);
        Vertex dest = (Vertex) id.getVertex(0);
        int result = UnweightedShortestPathUtil.getPathLen(source, dest, usp);
        assertEquals(-1, result);
        
        // [2] shortest path from 0 to 3 contains three vertices (3,4,0), len is 2
        source = (Vertex) id.getVertex(0);
        dest = (Vertex) id.getVertex(3);
        result = UnweightedShortestPathUtil.getPathLen(source, dest, usp);
        assertEquals(2, result);
        
        // [3] shortest path from 4 to 7 contains three vertices (7,1,0,4), len is 3
        source = (Vertex) id.getVertex(4);
        dest = (Vertex) id.getVertex(7);
        result = UnweightedShortestPathUtil.getPathLen(source, dest, usp);
        assertEquals(3, result);
    }
    
    
    /* See test Graph 7 above. */
    public void testGetPathMap () {
        System.out.println("testGetPathMap");
        
        UndirectedGraph ug = new UndirectedSparseGraph();
        GraphUtils.addVertices(ug,8);
        Indexer id = Indexer.getIndexer(ug);
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(0),(Vertex) id.getVertex(1));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(1),(Vertex) id.getVertex(2));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(2),(Vertex) id.getVertex(3));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(0),(Vertex) id.getVertex(4));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(4),(Vertex) id.getVertex(3));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(1),(Vertex) id.getVertex(7));
        
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(5),(Vertex) id.getVertex(6));
        UnweightedShortestPath usp = new UnweightedShortestPath(ug);
        
        //Set<Vertex> dest = new TreeSet<Vertex>();
        List<Vertex> dest = new ArrayList<Vertex>();
        dest.add((Vertex) id.getVertex(1));
        dest.add((Vertex) id.getVertex(4));
        dest.add((Vertex) id.getVertex(5));
        
        // <1> there is only one vertex from dist near 7. It is 1.
        Vertex source = (Vertex) id.getVertex(7);
        Map<Vertex,Integer> m = UnweightedShortestPathUtil.getPath(source, dest, usp);
        assertNotNull(m);
        assertEquals(1, m.keySet().size());
        assertTrue(m.containsKey((Vertex) id.getVertex(1)));
        assertEquals(1, (int)m.get((Vertex) id.getVertex(1)));
        
        // <2> there is one vertex from dist near 1. It is 4. len(1,0,4)=2
        source = (Vertex) id.getVertex(1);
        m = UnweightedShortestPathUtil.getPath(source, dest, usp);
        assertNotNull(m);
        assertEquals(1, m.keySet().size());
        assertTrue(m.containsKey((Vertex) id.getVertex(4)));
        assertEquals(2, (int)m.get((Vertex) id.getVertex(4)));
        
        // <3> there are two vertices from dist near 2. They are 1 (2,1) and 
        // 4 (2,3,4).
        source = (Vertex) id.getVertex(2);
        m = UnweightedShortestPathUtil.getPath(source, dest, usp);
        assertNotNull(m);
        assertEquals(2, m.keySet().size());
        assertTrue(m.containsKey((Vertex)   id.getVertex(1)));
        assertEquals(1, (int)m.get((Vertex) id.getVertex(1)));
        assertTrue(m.containsKey((Vertex)   id.getVertex(4)));
        assertEquals(2, (int)m.get((Vertex) id.getVertex(4)));
    }
    
}
