package matching;

import junit.framework.*;

import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.impl.SparseVertex;
import edu.uci.ics.jung.graph.impl.SparseGraph;
import edu.uci.ics.jung.utils.UserData;

import java.util.Map;
import java.util.HashMap;

public class VertexPairSimilarTest extends TestCase {
    
    public VertexPairSimilarTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
    }

    protected void tearDown() throws Exception {
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(VertexPairSimilarTest.class);
        
        return suite;
    }

    /**
     * Test of calcResult method, of class matching.VertexPairSimilar.
     */
    public void testCalcResult() {
        System.out.println("calcResult");
        
        Vertex v1, v2, v1_1, v2_1, v2_2;
        v1  = new SparseVertex();
        v2  = new SparseVertex();
        v1_1= new SparseVertex();
        v2_1= new SparseVertex();
        v2_2= new SparseVertex();
        
        int titles_sim = 3;
        int v1_neighbors = 11;
        int v2_neighbors = 18;
        Map<Vertex,Integer> v1_path_len = new HashMap<Vertex,Integer>();
        Map<Vertex,Integer> v1_path_len_real = new HashMap<Vertex,Integer>();
        v1_path_len.     put(v1_1, 22);
        v1_path_len_real.put(v1_1, 22);
        Map<Vertex,Integer> v2_path_len = new HashMap<Vertex,Integer>();
        Map<Vertex,Integer> v2_path_len_real = new HashMap<Vertex,Integer>();
        v2_path_len.put(v2_1, 33);
        v2_path_len.put(v2_2, 49);
        v2_path_len_real.put(v2_1, 33);
        v2_path_len_real.put(v2_2, 49);
        
        Map<Vertex,Vertex> similar_v1_v2 = new HashMap<Vertex,Vertex>();
        Map<Vertex,Vertex> similar_v2_v1 = new HashMap<Vertex,Vertex>();
        similar_v1_v2.put(v1_1, v2_1);
        similar_v2_v1.put(v2_1, v1_1);
        
        VertexPairSimilar pair = new VertexPairSimilar(v1, v2, 
                    titles_sim, similar_v1_v2, similar_v2_v1,
                    v1_neighbors, v2_neighbors, 
                    v1_path_len, v2_path_len,
                    v1_path_len_real, v2_path_len_real);
        
        double result = pair.getSimilarity();
        // 1 / (1 + 3 + (18-11) + ((33-22) + (49 - 0))) = 0.014084507
        double eps = 0.0001;
        assertTrue(VertexPairSimilar.equalsDoubles(0.014084507, result, eps));
        
        
        // test 2: when v1_path_len.size() > v2_path_len.size()
        Vertex v1_2, v1_3;
        v1_2= new SparseVertex();
        v1_3= new SparseVertex();
        v1_path_len.put(v1_2, 117);
        v1_path_len.put(v1_3, 133);
        similar_v1_v2.put(v1_2, v2_2);
        similar_v2_v1.put(v2_2, v1_2);
        assertTrue(v1_path_len.size() > v2_path_len.size());
        
        result = pair.calcTwoVertexSimilarity(similar_v1_v2, similar_v2_v1, v1_path_len_real, v2_path_len_real);
        // 1 / (1 + 3 + (18-11) + ((33-22) + (117 - 49) + (133 - 0))) = 0.004484305
        eps = 0.0001;
        assertTrue(VertexPairSimilar.equalsDoubles(0.004484305, result, eps));
    }
    
    public void testGetVertexByName() {
        System.out.println("testGetVertexByName and subcategory");
        
        Graph g = new SparseGraph();
        Vertex v1 = g.addVertex(new SparseVertex());
        Vertex v2 = g.addVertex(new SparseVertex());
        v1.addUserDatum(GraphComparator.NAME_KEY, "apple", UserData.SHARED);
        v2.addUserDatum(GraphComparator.NAME_KEY, "apple", UserData.SHARED);
        v1.addUserDatum(GraphComparator.SUBGRAPH_N, "1", UserData.SHARED);
        
        Vertex a = VertexPairSimilar.getVertexByName(g,"apple",null);
        Vertex b = VertexPairSimilar.getVertexByName(g,"apple","1");
        assertEquals(v1, b);
        assertEquals(v2, a);
        assertTrue(v1 != v2);
    }

    /**
     * Test of calcTwoVertexSimilarity method, of class matching.VertexPairSimilar.
     */ //testCalcSimilarity
    public void testCalcTwoVertexSimilarity() {
        System.out.println("calcTwoVertexSimilarity by GraphComparator.pair_similar");
        
        GraphCreator gd = new GraphCreator();
        Graph g1 = gd.createGraph(gd.labels1, gd.edges1);
        Graph g2 = gd.createGraph(gd.labels2, gd.edges2);
        
        // calculates initial similarity
        LevenshteinDistance lev = new LevenshteinDistance();
        lev.use_dist_matrix = true;
        lev.setContstraintType(LevenshteinDistance.TYPE_SOFT);
        
        UnweightedShortestPath usp1 = new UnweightedShortestPath(g1);
        UnweightedShortestPath usp2 = new UnweightedShortestPath(g2);
        
        GraphComparator gc = new GraphComparator();
        gc.compare(g1, g2, lev);
        
        Vertex v1, v2;
        VertexPairSimilar pair;
        
        {
            // pair (Thing,Thing) levenshtein_distance=0
            v1 = VertexPairSimilar.getVertexByName(g1, "Thing");
            v2 = VertexPairSimilar.getVertexByName(g2, "Thing");
            pair = gc.pair_similar.get(v1);
            assertEquals(0, pair.string_sim);
            assertEquals(1, pair.v1_neighbours);
            assertEquals(1, pair.v2_neighbours);
            assertTrue(VertexPairSimilar.equalsDoubles(pair.getSimilarity(), 0.5));
        }
        
        {
            // pair (Supplier,Computer) levenshtein_distance=5
            v1 = VertexPairSimilar.getVertexByName(g1, "Supplier");
            v2 = VertexPairSimilar.getVertexByName(g2, "Computer");
            
            pair = gc.pair_similar.get(v1);
            assertEquals(5, pair.string_sim);
            assertEquals(3, pair.v1_neighbours);
            assertEquals(2, pair.v2_neighbours);
            assertTrue(VertexPairSimilar.equalsDoubles(pair.getSimilarity(), 1.0/8.0));
        }
        
        {
            // pair (Furniture,Furniture) levenshtein_distance=0
            v1 = VertexPairSimilar.getVertexByName(g1, "Furniture");
            v2 = VertexPairSimilar.getVertexByName(g2, "Furniture");
            
            Map<Vertex,Integer> v1_path_len = UnweightedShortestPathUtil.getPath(v1, gc.similar_v1_v2.keySet(), usp1);
            Map<Vertex,Integer> v2_path_len = UnweightedShortestPathUtil.getPath(v2, gc.similar_v2_v1.keySet(), usp2);
            Map<Vertex,Integer> v1_path_len_real = UnweightedShortestPathUtil.getPathReal(v1, gc.similar_v1_v2.keySet(), usp1);
            Map<Vertex,Integer> v2_path_len_real = UnweightedShortestPathUtil.getPathReal(v2, gc.similar_v2_v1.keySet(), usp2);
            
            pair = new VertexPairSimilar(v1, v2,
                    //usp1, usp2,
                    (Integer)v1.getUserDatum(GraphComparator.SIM_KEY), 
                    gc.similar_v1_v2,   gc.similar_v2_v1,
                    v1.numNeighbors(),  v2.numNeighbors(), 
                    v1_path_len,        v2_path_len,
                    v1_path_len_real,   v2_path_len_real);
            
            //pair = gc.pair_similar.get(v1);
            assertEquals(0, pair.string_sim);
            assertEquals(1, pair.v1_neighbours);
            assertEquals(2, pair.v2_neighbours);
            assertTrue(VertexPairSimilar.equalsDoubles(pair.getSimilarity(), 0.5));
        }
        
        // test the case when there are more than one shortes path, 
        // but shortes path in graph g3 and g4 have equal length
        //      checked function: UnweightedShortestPathUtil.getPathLen(source, dest, usp);
        for(int i=0;i<10;i++)
        {   
            GraphCreator creator = new GraphCreator();
            Graph g3 = creator.createGraph(creator.labels3, creator.edges3);
            Graph g4 = creator.createGraph(creator.labels4, creator.edges4);
            gc.compare(g3, g4, lev);
            
            v1 = VertexPairSimilar.getVertexByName(g3, "node3");
            v2 = VertexPairSimilar.getVertexByName(g4, "node3");
            
            UnweightedShortestPath usp3 = new UnweightedShortestPath(g3);
            UnweightedShortestPath usp4 = new UnweightedShortestPath(g4);
            Map<Vertex,Integer> v1_path_len = UnweightedShortestPathUtil.getPath(v1, gc.similar_v1_v2.keySet(), usp3);
            Map<Vertex,Integer> v2_path_len = UnweightedShortestPathUtil.getPath(v2, gc.similar_v2_v1.keySet(), usp4);
            Map<Vertex,Integer> v1_path_len_real = UnweightedShortestPathUtil.getPathReal(v1, gc.similar_v1_v2.keySet(), usp3);
            Map<Vertex,Integer> v2_path_len_real = UnweightedShortestPathUtil.getPathReal(v2, gc.similar_v2_v1.keySet(), usp4);
            
            pair = new VertexPairSimilar(v1, v2, 
                    //usp3, usp4,
                    (Integer)v1.getUserDatum(GraphComparator.SIM_KEY), 
                    gc.similar_v1_v2,   gc.similar_v2_v1,
                    v1.numNeighbors(),  v2.numNeighbors(), 
                    v1_path_len,        v2_path_len,
                    v1_path_len_real,   v2_path_len_real);
            
            //pair = gc.pair_similar.get(v1);
            assertEquals(0, pair.string_sim);
            assertEquals(1, pair.v1_neighbours);
            assertEquals(2, pair.v2_neighbours);
            System.out.println(pair.getSimilarity());
            assertTrue(VertexPairSimilar.equalsDoubles(pair.getSimilarity(), 0.5));
        }    
    }
    
    public void testCalcSimilarity() {
        System.out.println("calcSimilarity");
        
        GraphCreator gd = new GraphCreator();
        Graph g1 = gd.createGraph(gd.labels1, gd.edges1);
        Graph g2 = gd.createGraph(gd.labels2, gd.edges2);
        
        // calculates initial similarity
        LevenshteinDistance lev = new LevenshteinDistance();
        lev.use_dist_matrix = true;
        lev.setContstraintType(LevenshteinDistance.TYPE_SOFT);
        
        GraphComparator gc = new GraphComparator();
        double r = gc.compare(g1, g2, lev);
        assertTrue(VertexPairSimilar.equalsDoubles(r, 0.25)); // 0.138888888
    }
    
    
}
